/****************************************************************************/
/*  lnk.cmd   v#####                                                        */
/*  Copyright (c) 1996@%%%%  Texas Instruments Incorporated                 */
/*    Usage:  lnk6x <obj files...>    -o <out file> -m <map file> lnk.cmd   */
/*            cl6x  <src files...> -z -o <out file> -m <map file> lnk.cmd   */
/*                                                                          */
/*    Description: This file is a sample linker command file that can be    */
/*                 used for linking programs built with the C compiler and  */
/*                 running the resulting .out file on a C672x               */
/*                 simulator.  Use it as a guideline.  You will want to     */
/*                 change the memory layout to match your specific C6xxx    */
/*                 target system.  You may want to change the allocation    */
/*                 scheme according to the size of your program.            */
/*                                                                          */
/*    Notes: (1)   You must specify a directory in which rts6x00.lib is     */
/*                 located.  either add a -i"<directory>" line to this      */
/*                 file or use the system environment variable C_DIR to     */
/*                 specify a search path for the libraries.                 */
/*                                                                          */
/*           (2)   If the run-time library you are using is not named       */
/*                 rts6200[e].lib, rts6400[e].lib, or rts6700[e].lib, be    */
/*                 sure to use the correct name here.                       */
/*                                                                          */
/****************************************************************************/

/* SPECIFY THE SYSTEM MEMORY MAP */

/*===========================================================================*/
/* Memory Map                                                                */
/*   This linker command file was created to provide a general guideline     */
/*   for linking code with the secondary boot loader. Some of the memory     */
/*   section listed below may not be required for all applications. Please   */
/*   read this section to help determine which memory allocation sections    */
/*   may apply to your application.                                          */
/*                                                                           */
/*  IROM_DSPLIB:    -  Required only if accessing ROM Contents for DSPLIB    */
/*  IROM_FASTRTS:   -  Required only if accessing ROM Contents for FastRts   */
/*  IROM_BIOS:      -  Required only if accessing ROM Contents for DSPBIOS   */     
/*  IRAM_BOOT:      -  Reserved at boot - application may use this space for */
/*                     uninitialized data/code sections                      */
/*  IRAM_BIOS:      -  Reserved for ROMed version of DSPBIOS, may be used by */
/*                  -  application if ROM contents for DSPBIOS is unused     */
/*  IRAM_RESERVED:  -  Reserved for ROM contents, may be used by application */
/*                    if ROMed DSPLIB and FastRts are unused                 */
/*  VEC:            -  May be placed on any on 8 byte boundary               */
/*  RAM:            -  Internal RAM available for aplication code use        */
/*  ASYNC2:         -  External EMIFA address space, This space will be used */
/*                     to place the AIS dat stream used during FLASH boot    */
/*  SDRAM:          -  May be used as needed by application                  */
/*===========================================================================*/


MEMORY
{
    IROM_DSPLIB:     o = 00020000h   l = 0000C000h
    IROM_FASTRTS:    o = 0002C000h   l = 00004000h
    IROM_BIOS:       o = 00030000h   l = 00030000h     
    IRAM_BOOT:       o = 10000000h   l = 00001000h
    IRAM_BIOS:       o = 10001000h   l = 00000B00h
    IRAM_RESERVED:   o = 10001B00h   l = 00000100h
    IRAM_STACK:      o = 10001C00h   l = 00000800h
    VEC:             o = 10002400h   l = 00000A00h
    RAM:             o = 10002E00h   l = 0002A000h
    ASYNC2:          o = 90000000h   l = 00008000h
    SDRAM:           o = 80000000h   l = 08000000h
}


SECTIONS
{
    

  /*-------------------------------------------------------------*/
  /* Sample allocation for Compiler generated sections that may  */
  /* be defined in application code.                             */
  /*-------------------------------------------------------------*/
    .stack       >       RAM, fill = 0xdeadbeef
    .bss         >       RAM
    .cinit       >       RAM
    .pinit       >       RAM
    .cio         >       RAM 
    .const       >       RAM
    .data        >       RAM
    .switch      >       RAM 
    .sysmem      >       SDRAM
    .text        >       RAM
    .far         >       RAM

                              
}                             